/* Copyright (c) 2019 Atmark Techno, Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <modbus.h>

#define SERIAL_SPEED 19200
#define SERIAL_PARITY 'N'
#define SERIAL_DATABIT 8
#define SERIAL_STOPBIT 1
#define SLAVE_ADDRESS 3

void usage(char *prog_name)
{
	printf("Usage: %s <device> <led number> <on/off>\n\n", prog_name);
}

int main(int argc, char *argv[])
{
	modbus_t *ctx;

	uint8_t *send_req;

	/* 送信パケット */
	uint8_t send_packet[] = {0x03, 0x06, 0x00, 0x04, 0x00, 0x00};

	uint8_t rsp_reg[25];
	int ret = EXIT_SUCCESS;
	int req_len;
	int rsp_len;
	char *endptr;

	long led_number;
	static char *serial_device_name;

	if (argc <= 3)
	{
		usage(argv[0]);
		return EXIT_FAILURE;
	}

	serial_device_name = argv[1];

	led_number = strtol(argv[2], &endptr, 10);
	if (led_number < 0 || led_number > 255 || *endptr)
	{
		usage(argv[0]);
		return EXIT_FAILURE;
	}

	send_packet[3] = (uint8_t)led_number;

	if (!strcmp(argv[3], "on"))
	{
		send_packet[5] = 0x01;
	}
	else if (!strcmp(argv[3], "off"))
	{
		send_packet[5] = 0x00;
	}
	else
	{
		usage(argv[0]);
		return EXIT_FAILURE;
	}

	/* RTUのlibmodbusコンテキストを作成 */
	ctx = modbus_new_rtu(serial_device_name, SERIAL_SPEED, SERIAL_PARITY, SERIAL_DATABIT, SERIAL_STOPBIT);
	if (!ctx)
	{
		fprintf(stderr, "Failed to create the context: %s\n", modbus_strerror(errno));
		return EXIT_FAILURE;
	}

	/* modbusの動作を確認できるデバッグ機能 */
	modbus_set_debug(ctx, TRUE);

	if (modbus_connect(ctx) == -1)
	{
		fprintf(stderr, "Unable to connect: %s\n", modbus_strerror(errno));
		modbus_free(ctx);
		return EXIT_FAILURE;
	}

	req_len = modbus_send_raw_request(ctx, send_packet, sizeof(send_packet));
	if (req_len < 0)
	{
		printf("request error.\n");
		fprintf(stderr, "faild: %s\n"
						"modbus error\n",
				modbus_strerror(errno));
		modbus_close(ctx);
		modbus_free(ctx);
		return EXIT_FAILURE;
	}

	rsp_len = modbus_receive_confirmation(ctx, rsp_reg);
	if (rsp_len < 0)
	{
		printf("confirmation error.\n");
		fprintf(stderr, "faild: %s\n"
						"modbus error\n",
				modbus_strerror(errno));
		ret = EXIT_FAILURE;
	}

	modbus_close(ctx);
	modbus_free(ctx);

	return ret;
}
